unit Cal; // Create this form third (after Debug and Comm)
// 1/14/06 - Added dynamic range labels in Refresh
// 2/6/06 - Centered form on screen
// 2/3/07 - Rewritten with OOP, moved StartCal, ReceiveCal, SendCal to OrtComm
// 3/7/09 - Added date field
// 3/8/09 - Added try..except for file I/O
// 3/18/09 - Removed TEMPFILE, Changed CalRecord, Added adjust All/Selected
// 12/22/09 - Make universal with define of BTSA or ORTM
// 4/1/10 - Removed multiple Read/Write/Get Cal

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Comm, USB, Setup,
  StdCtrls, ExtCtrls, Spin,
  Debug, Waveform, Delays;

const
{$IFDEF BTSA}
  CALFILE = 'BTSA3.ocf';
{$ELSE}
  CALFILE = 'Ortmaster.ocf';
{$ENDIF}

type
  TCal = record
    Factor: array [1..8] of single;
    OnThresh: Integer;
    MPUThresh: Integer;
    OnDelay: Integer;
    OffThresh: Integer;
    OffDelay: Integer;
    AverageTime: Real;
    SerNum: string;
    Date: Tdate;
end;

type
  TCalRecType = (PICdata, PICarray);
  TCalRecord = record
  case TCalRecType of
    PICdata: (
    Factor: array [1..8] of array[0..9] of char;  //80
    OnThresh: array[0..3] of char;                //+4=84
    MPUThresh: array[0..4] of char;               //+5=89
    OnDelay: array[0..3] of char;                 //+4=93
    OffThresh: array[0..3] of char;               //+4=97
    OffDelay: array[0..3] of char;                //+4=101
    AverageTime: array[0..7] of char;             //+8=109
    SerNum: array[0..11] of char;                 //+12=121
    Date: array[0..6] of char                    //+7=128   5 digits 0-'99999'
    );
  PICarray:
    (Data: Array[0..127] of char);
end;

type
  TfmCal = class(TForm)
    Label1: TLabel;
    CalFactor1: TEdit;
    Label2: TLabel;
    CalFactor2: TEdit;
    Label3: TLabel;
    CalFactor3: TEdit;
    Label4: TLabel;
    CalFactor4: TEdit;
    Label5: TLabel;
    CalFactor5: TEdit;
    Label6: TLabel;
    CalFactor6: TEdit;
    Label7: TLabel;
    CalFactor7: TEdit;
    Label8: TLabel;
    CalFactor8: TEdit;
    SpinBtnAdjust: TSpinButton;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label19: TLabel;
    Label20: TLabel;
    CalSerNum: TEdit;
    SaveButton: TButton;
    Label13: TLabel;
    CalOnThresh: TSpinEdit;
    CalOnDelay: TSpinEdit;
    CalOffThresh: TSpinEdit;
    CalOffDelay: TSpinEdit;
    Label14: TLabel;
    CalMPUThresh: TSpinEdit;
    Label15: TLabel;
    CalDate: TEdit;
    RadioGroup1: TRadioGroup;
    btToday: TButton;
    CalAvgTime: TComboBox;
    procedure CalOffDelayChange(Sender: TObject);
    procedure CalOffThreshChange(Sender: TObject);
    procedure CalOnDelayChange(Sender: TObject);
    procedure CalOnThreshChange(Sender: TObject);
    procedure CalMPUThreshChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure OnActivate(Sender: TObject);
    procedure SaveButtonClick(Sender: TObject);
    procedure SpinBtnAdjustDownClick(Sender: TObject);
    procedure SpinBtnAdjustUpClick(Sender: TObject);
    procedure btTodayClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure CalAvgTimeChange(Sender: TObject);
  private
    CalRecord: TCalRecord;
    procedure  ReadCalRecord;
    procedure  RefreshCal;
  public
    Cal: TCal;
    CalError:  Boolean;
    MaxSample: Integer;
    function   GetCalData: Boolean;             // Read from PIC
    procedure  ReadCalFile;
    procedure  WriteCalFile;
    procedure  SetDefaultCal;
  end;

var
  fmCal: TfmCal;

implementation

{$R *.DFM}

uses
{$IFDEF BTSA}
  BTSA3Main;
{$ELSE}
  OrtMain;
{$ENDIF}


//************************** CalMPUThreshChange ****************************
procedure TfmCal.CalMPUThreshChange(Sender: TObject);
begin
  cal.MPUThresh := CalMPUThresh.Value;
end;

//************************** CalOnThreshChange ****************************
procedure TfmCal.CalOnThreshChange(Sender: TObject);
begin
  cal.OnThresh := CalOnThresh.Value;
end;

//************************** CalOffDelayChange ****************************
procedure TfmCal.CalOffDelayChange(Sender: TObject);
begin
  cal.OffDelay := CalOffDelay.Value;
end;

//************************** CalOffThreshChange ****************************
procedure TfmCal.CalOffThreshChange(Sender: TObject);
begin
  cal.OffThresh := CalOffThresh.Value;
end;

//************************** CalOnDelayChange ****************************
procedure TfmCal.CalOnDelayChange(Sender: TObject);
begin
  cal.OnDelay := CalOnDelay.Value;
end;

//****************************** FormClose ********************************
procedure TfmCal.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  fmDebug.AddDebugItem( 'fmCal.OnClose ' );
end;

//************************** GetCalData ****************************
function TfmCal.GetCalData: Boolean;             // Read from PIC
var      S: Array[0..127] of char;
begin
  if (fmSetup.Source = 'USB') then
    Result := fmUSB.USBReceiveCal( S )
  else if (fmSetup.Source = 'COM') then
    Result := fmComm.ReceiveCal( S )
  else begin
    ReadCalFile;
    Result := True;
    end;
  CalError := True;
  If Result = True then begin
    StrLCopy(CalRecord.Data, S, 128);
    ReadCalRecord;
    CalError := False;
    end
  else
    SetDefaultCal;
end;

//************************** OnActivate ****************************
procedure    TfmCal.OnActivate(Sender: TObject);
begin
  fmDebug.AddDebugItem( 'fmCal.OnActivate ' );
  if not Demo then
    GetCalData
  else
    ReadCalFile;
  RefreshCal;     // Update values on screen
end;

//************************** RefreshCal ****************************
// Update edit boxes with actual calibration variables for editing
procedure TfmCal.RefreshCal;
var
  S: string;
begin
  fmDebug.AddDebugItem( 'RefreshCal ' );
{$IFDEF BTSA}
  with fmBTSA3 do begin
{$ELSE}
  with fmOrt do begin
{$ENDIF}
  Label1.Caption := RangeText[1];  // Added 1/14/06
  Label2.Caption := RangeText[2];
  Label3.Caption := RangeText[3];
  Label4.Caption := RangeText[4];
  Label5.Caption := RangeText[5];
  Label6.Caption := RangeText[6];
  Label7.Caption := RangeText[7];
  Label8.Caption := RangeText[8];
  end;
  str(cal.Factor[1]:8:6, S);
  CalFactor1.text := S;
  str(cal.Factor[2]:8:6, S);
  CalFactor2.text := S;
  str(cal.Factor[3]:8:6, S);
  CalFactor3.text := S;
  str(cal.Factor[4]:8:6, S);
  CalFactor4.text := S;
  str(cal.Factor[5]:8:6, S);
  CalFactor5.text := S;
  str(cal.Factor[6]:8:6, S);
  CalFactor6.text := S;
  str(cal.Factor[7]:8:6, S);
  CalFactor7.text := S;
  str(cal.Factor[8]:8:6, S);
  CalFactor8.text := S;
  CalOnThresh.Value := cal.OnThresh;
  CalMPUThresh.Value := cal.MPUThresh;
  CalOnDelay.Value := cal.OnDelay;
  CalOffThresh.Value := cal.OffThresh;
  CalOffDelay.Value := cal.OffDelay;
  str(cal.AverageTime:4:3, S);
  CalAvgTime.text := S;
  CalSerNum.text := cal.SerNum;
  CalDate.text := DateToStr(cal.Date);

end;

procedure  TfmCal.ReadCalRecord;
var        S: String;
           i: Integer;
begin
    try //try 2
        CalFactor1.text := Trim(CalRecord.Factor[1]);
        CalFactor2.text := Trim(CalRecord.Factor[2]);
        CalFactor3.text := Trim(CalRecord.Factor[3]);
        CalFactor4.text := Trim(CalRecord.Factor[4]);
        CalFactor5.text := Trim(CalRecord.Factor[5]);
        CalFactor6.text := Trim(CalRecord.Factor[6]);
        CalFactor7.text := Trim(CalRecord.Factor[7]);
        CalFactor8.text := Trim(CalRecord.Factor[8]);
        CalAvgTime.text := Trim(CalRecord.AverageTime);
        CalSerNum.text := Trim(CalRecord.SerNum);
        S := Trim(CalRecord.Date);
        SetLength( S, 5 );
        CalDate.text := DateToStr(StrToInt(S));

        for i := 1 to 8 do begin //1
          S := Trim(CalRecord.Factor[i]);
          SetLength( S, 8 );
          cal.Factor[i] := StrToFloat( S );
          end; //-1
        S:=  Trim(CalRecord.OnThresh);
        SetLength( S, 2 );
        cal.OnThresh := StrToInt( S );
        S := Trim(CalRecord.MPUThresh);
        SetLength( S, 3 );
        cal.MPUThresh := StrToInt( S );
        S := Trim(CalRecord.OnDelay);
        SetLength( S, 2 );
        cal.OnDelay := StrToInt( S );
        S := Trim(CalRecord.OffThresh);
        SetLength( S, 2 );
        cal.OffThresh := StrToInt( S );
        S := Trim(CalRecord.OffDelay);
        SetLength( S, 2 );
        cal.OffDelay := StrToInt( S );
        S := Trim(CalRecord.AverageTime);
        SetLength( S, 6 );
        cal.AverageTime := StrToFloat( S );
        if fmWaveform.ADCrate <= 0 then
          fmWaveform.ADCrate := 2400.0;
        MaxSample := round(fmWaveform.ADCrate * cal.AverageTime);
        S := Trim(CalRecord.SerNum) + '            '; // Pad with 12 spaces
        SetLength( S, 10 );
        cal.SerNum := S;
        S:= Trim(CalRecord.Date)+ '     ';
        SetLength( S, 5 );
        cal.Date := StrToInt( S );
        CalError := False;
      except // try 2
        on EConvertError do
          MessageDlg('Calibration corrupted. Please recalibrate.',mtInformation,[mbOK], 0);
        else
          MessageDlg('Calibration corrupted. Please recalibrate.',mtInformation,[mbOK], 0);
        CalError := True;
        SetDefaultCal;
      end; // try 2
end;

//************************** ReadCalFile ****************************
// Reads values from cal file into Cal Record
// Sets CalError if CalFile not found
// Sets CalError if any text does not convert to variable type
// Calls SetDefaultCal if CalError
procedure TfmCal.ReadCalFile;
var
  fsCalFile: TFileStream;
  Bytes: Integer;

begin
  fmDebug.AddDebugItem( 'ReadCalFile ' );
  SetCurrentDir(ProgramDataFolder);  // added in case dir changed
  if not FileExists( CALFILE ) then begin //1
    MessageDlg('Cal File ' + CALFILE + ' Missing.',mtInformation,[mbOK], 0);
    SetDefaultCal; // Creates and writes new CALFILE with default data
    end; //-1
  fsCalFile := TFileStream.Create(CALFILE, fmOpenRead);
  try //try 1
    Bytes := fsCalFile.Read( CalRecord.Data, sizeof(CalRecord.Data) );
    ReadCalRecord; // Sets CalError
  except // try 1
    MessageDlg('Error reading file: '+ CALFILE, mtError, [mbAbort], 0);
    CalError := True;
  end; // try 1
  fsCalFile.Free;
  fmDebug.AddDebugItem( 'End ReadCalFile ' );
end;

//************************** SaveButtonClick ****************************
// Update actual cal variables with edited information in Calibration Menu
// Write to cal file
procedure TfmCal.SaveButtonClick(Sender: TObject);
var       S : String;
begin
  fmDebug.AddDebugItem( 'frmCalibration.SaveButtonClick ' );
  cal.Factor[1] := StrToFloat( CalFactor1.text );
  cal.Factor[2] := StrToFloat( CalFactor2.text );
  cal.Factor[3] := StrToFloat( CalFactor3.text );
  cal.Factor[4] := StrToFloat( CalFactor4.text );
  cal.Factor[5] := StrToFloat( CalFactor5.text );
  cal.Factor[6] := StrToFloat( CalFactor6.text );
  cal.Factor[7] := StrToFloat( CalFactor7.text );
  cal.Factor[8] := StrToFloat( CalFactor8.text );
  cal.OnThresh := CalOnThresh.Value;
  cal.MPUThresh := CalMPUThresh.Value;
  cal.OnDelay := CalOnDelay.Value;
  cal.OffThresh := CalOffThresh.Value;
  cal.OffDelay := CalOffDelay.Value;
  cal.AverageTime := StrToFloat( CalAvgTime.text );
  S := CalSerNum.Text + '            ';
  SetLength(S, 10);
  cal.SerNum := S;
  cal.Date := StrToDate( CalDate.Text );
  WriteCalFile;          // Writes cal.x to file
  ReadCalFile;           // Reads file into CalRecord (Text) and update cal.x
  if (fmSetup.Source = 'USB') then begin
    if fmUSB.USBConnected(ORTM4_VID_PID) then
      fmUSB.USBSendCal(CalRecord.Data);              // Delay Built in to procedure
    end
  else if (fmSetup.Source = 'COM') then begin
    if fmComm.CommOK then
      fmComm.SendCal(CalRecord.Data);              // Delay Built in to procedure
    end;
//  ReadCalFile;        // Added 3/28/05 to update changes
//  GetCalData;         // Includes ReadCal
  Close;
  fmDebug.AddDebugItem( 'End frmCalibration.SaveButtonClick ' );
end;

//************************** SetDefaultCal ****************************
procedure TfmCal.SetDefaultCal;
var i: SmallInt;
    S: String;
begin
  fmDebug.AddDebugItem( 'SetDefaultCal ' );
  S := 'Setting Default Values ';
  CalError := True;
  if( MessageDlg( S, mtInformation, [mbOK,mbCancel], 0)) = mrOK
    then begin
    for i := 1 to 8 do
      cal.Factor[i] := 0.001225;
    cal.OnThresh := 40;
    cal.MPUThresh := 100;
    cal.OnDelay := 5;
    cal.OffThresh := 20;
    cal.OffDelay := 15;
    cal.AverageTime := 0.100;
    cal.SerNum := '**DEFAULT*'; // 10 characters
    cal.Date := Date; // Date format
    WriteCalFile;
    RefreshCal;
  end;
  fmDebug.AddDebugItem( 'End SetDefaultCal ' );
end;

//*********************** SpinBtnAdjustDownClick *************************
procedure TfmCal.SpinBtnAdjustDownClick(Sender: TObject);
var       n:Integer;
begin
  if RadioGroup1.ItemIndex = 0 then begin
    for n := 1 to 8 do
      if cal.Factor[n] > 0.001000 then
        cal.Factor[n] := cal.Factor[n] - 0.000001;
    end
  else begin
    if CalFactor1.Focused then
      n := 1
    else if CalFactor2.Focused then
      n := 2
    else if CalFactor3.Focused then
      n := 3
    else if CalFactor4.Focused then
      n := 4
    else if CalFactor5.Focused then
      n := 5
    else if CalFactor6.Focused then
      n := 6
    else if CalFactor7.Focused then
      n := 7
    else if CalFactor8.Focused then
      n := 8
    else
      n := 0;
    if n > 0 then
      if cal.Factor[n] > 0.001000 then
        cal.Factor[n] := cal.Factor[n] - 0.000001;
    end;
  RefreshCal;
end;

//*********************** SpinBtnAdjustUpClick *************************
procedure TfmCal.SpinBtnAdjustUpClick(Sender: TObject);
var       n:Integer;
begin
  if RadioGroup1.ItemIndex = 0 then begin
    for n := 1 to 8 do
      if cal.Factor[n] < 0.002000 then
        cal.Factor[n] := cal.Factor[n] + 0.000001;
    end
  else begin
    if CalFactor1.Focused then
      n := 1
    else if CalFactor2.Focused then
      n := 2
    else if CalFactor3.Focused then
      n := 3
    else if CalFactor4.Focused then
      n := 4
    else if CalFactor5.Focused then
      n := 5
    else if CalFactor6.Focused then
      n := 6
    else if CalFactor7.Focused then
      n := 7
    else if CalFactor8.Focused then
      n := 8
    else
      n := 0;
    if n > 0 then
      if cal.Factor[n] < 0.002000 then
        cal.Factor[n] := cal.Factor[n] + 0.000001;
    end;
  RefreshCal;
end;

//************************** WriteCalFile ****************************
// Writes actual cal data to cal file
procedure TfmCal.WriteCalFile;

var
  cal_file:  TextFile;
  i: SmallInt;
  S: string;
  Attr: Integer;
begin
  fmDebug.AddDebugItem( 'WriteCalFile' );
  S := 'Writing to Cal File '+ CALFILE;
  if( MessageDlg( S, mtInformation, [mbOK,mbCancel], 0)) = mrOK
    then begin
    try
      if ( FileExists( CALFILE ) ) then begin //1
        Attr := FileGetAttr( CALFILE );
        Attr := Attr and not(faReadOnly);
        FileSetAttr( CALFILE, Attr ); end
      else
        Attr := 0;
      AssignFile( cal_file, CALFILE );
      rewrite(cal_file);
      for i := 1 to 8 do begin
        Str(cal.Factor[i]:8:6, S );
        writeln(cal_file, S ); end;       // 8 * (8+2) = 80 bytes
      Str(cal.OnThresh:2, S );
      writeln(cal_file, S );              // +4 = 84
      Str(cal.MPUThresh:3, S );
      writeln(cal_file, S );              // +5 = 89
      Str(cal.OnDelay:2, S );
      writeln(cal_file, S );              // +4 = 93
      Str(cal.OffThresh:2, S );
      writeln(cal_file, S );              // +4 = 97
      Str(cal.OffDelay:2, S );
      writeln(cal_file, S );              // +4 = 101
      Str(cal.AverageTime:6:3, S );
      writeln(cal_file, S );              // +6 +2 = 109
      S:= Cal.SerNum + '          ';
      SetLength(S,10);
      writeln(cal_file, S );     // +12 +2 = 121
      S:= IntToStr(Round(cal.Date));    // TDateTime is Double - use Round
      SetLength(S,5);
      writeln(cal_file, S );     // +5 +2 = 128
      FileSetAttr( CALFILE, Attr or faReadOnly );
      closefile(cal_file);
    except
      MessageDlg('Error writing file: '+CALFILE, mtError, [mbAbort], 0);
    end;
  end;
  fmDebug.AddDebugItem( 'End WriteCalFile' );
end;

procedure TfmCal.btTodayClick(Sender: TObject);
begin
  cal.Date := Now;
  RefreshCal;
end;

procedure TfmCal.FormShow(Sender: TObject);
begin
  fmDebug.AddDebugItem( 'fmCal.OnShow ' );
//  if fmComm.CommOK then
//    GetCalData
//  else
//    ReadCalFile;
//  RefreshCal;     // Update values on screen
end;

procedure TfmCal.CalAvgTimeChange(Sender: TObject);
begin
  cal.AverageTime := StrToFloat(CalAvgTime.Text);
  RefreshCal;     // Update values on screen
end;

end.
