unit Comm; // Create this form second

interface // Declarations available to clients (Forward declarations)

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  CommPortList, Debug, Delays,
  SerialNG, StdCtrls, ExtCtrls, syncobjs, Registry;

const
  MAXCOMM = 100000;
  RXQUEUE = 48000;
  E_FIRSTCHAR = $01;
  E_SECONDCHAR = $02;
  E_CHARMATCH = $04;
  E_BUFFERMAX = $08;
  Ctrl_A = $01;  //Start Transmission
  Ctrl_C = $03;  //Return Calibration Data in Flash
  Ctrl_D = $04;  //Stop Transmission
  Ctrl_E = $05;  //Sending Calibration Data to Flash
  Ctrl_T = $14;  //Test
  CR = char($0d);
  LF = char($0a);
  DEVID = 'Vid_04d8&Pid_fdcf';
  DEVSN1 = '0123456789';
  DEVSN2 = 'ETI20090414';
  DEVDESC1 = 'Ortmaster';
  DEVDESC2 = 'BTSA-3';

type  TCommPortStatus = (sOK, sOpenError, sTimeout, sCharError, sNone);

type  TCommPortInfo = record
        FriendlyName: String;
        PortName: String;
        end;

//**************************** fmComm Object ********************************
type  // Note: all of these items are listed as Published
  TfmComm = class(TForm)
    BtnClear: TButton;
    CommErrorBox: TEdit;
    CommTimer: TTimer;
    ErrorCountBox: TEdit;
    ErrorCountLabel: TLabel;
    ErrorListBox1: TListBox;
    ErrorMaskBox: TEdit;
    HideButton: TButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    lbCommError: TLabel;
    MaxRxDbox: TEdit;
    RxDCountBox: TEdit;
    BtnPorts: TButton;
    procedure BtnClearClick(Sender: TObject);
    procedure CommTimerTimer(Sender: TObject);
    procedure ErrorMaskBoxChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure HideButtonClick(Sender: TObject);
    procedure BtnPortsClick(Sender: TObject);

  private
    CommBufferPtr: Integer;       // Used in ReadCluster
    CommError: Byte;           // CommErrorType = eNone;
    CommReadPtr: Integer;         // Used in ReadBuffer
    ElapsedTime: LongWord;
    ErrorMask: Integer;
    LastRxError: Integer;
    RxDcount: Integer;      // Bytes in Main Comm Buffer
    procedure ReadCluster( Sender: TObject );

  public  // Note: procedures with () not listed as Public
    CommBuffer: Array[0..MAXCOMM] of Char;   //Raw Comm Buffer
    CommOK: Boolean;
    CommCharCount: Integer; // Bytes in CommBuffer waiting to be processed by ReadBuffer
    CommPortName: String;
    FirstChar, SecondChar: Char;
    MaxRxDcount: Integer;
    RxDerrorCount: Integer;
    RxError: Integer;
    Sampling: Boolean;
    TestedPort:   String;
    procedure ClearInput(ShowDebug: Boolean);
    function  GetCommPortName: String; // stdcall;
    function  OpenComm( name: String ): Boolean;
    procedure ProcessError(Sender: TObject; Place,
      Code: Cardinal; Msg: String; Noise: Byte);
    function  ReadBuffer : Integer;
    function  ReadTestString : Boolean;
    function  ReceiveCal( Var sData: Array of char ): Boolean;
    procedure SendCal( Data: Array of char );
    procedure SendRange( RangeNum: Integer );
    function  SerialPortON(ComPort: String): Boolean;
    procedure SerialPortOFF;
    procedure StartComm;
    procedure StopComm;
    procedure SendChar( ch: char );
    procedure SendStr( S: String );
    function  TestComm( name: String ): TCommPortStatus;

  end;

//**************************** Global Variables ********************************
var
  fmComm: TfmComm;
  CommSerialPort: TSerialPortNG;
  CommCriticalSection: TCriticalSection;
  CommPortInfo: TCommPortInfo;

function   GetCommPortInfo( id: String; sn: String ): TCommPortInfo;

implementation  // Definition of functions and procedures declared in interface
                // May be followed by initialization and finalization sections

{$R *.DFM}

//************************ BtnClearClick *******************************
procedure TfmComm.BtnClearClick(Sender: TObject);
begin
  RxError := 0;
  LastRxError := 0;
  MaxRxDcount := 0;
end;

//************************ ClearInput *******************************
procedure TfmComm.ClearInput(ShowDebug: Boolean);
var       Timeout: Integer;
          n: Integer;
begin
  if ShowDebug then
    fmDebug.AddDebugItem( 'ClearInput ' );
  Timeout := 10;
  if (CommSerialPort = nil) then
    exit;
  While Timeout > 0 do begin
    if CommSerialPort.SendInProgress then begin
      fmDebug.AddDebugItem( 'ClearInput Delay ' );
      Delay(100);
      Dec( Timeout );
      end
    else
      break;
  end;
  DelayMsec(100);
  CommSerialPort.ClearRxdQueue;
  for n := MAXCOMM downto 0 do
    CommBuffer[n] := char(0);
  CommBufferPtr := 0;
  CommReadPtr := 0;
  CommCharCount := 0;
  RxDcount := CommSerialPort.NextClusterSize;
  if RxDcount < 0
    then RxDcount := 0;

end;

//************************ CommTimerTimer *******************************
// This is used to report Comm Status and Errors
procedure TfmComm.CommTimerTimer(Sender: TObject);
begin
  ElapsedTime := ElapsedTime + CommTimer.Interval;
  If CommSerialPort = nil then
    exit;
  If CommSerialPort.Active and Sampling then begin
    RxDCountBox.Text := Format( '%d', [RxdCount] );
    ErrorCountBox.Text := Format( '%d', [RxError] );
    CommErrorBox.Text := Format( '%x', [CommSerialPort.CommError] );
    MaxRxDbox.Text := Format( '%d', [MaxRxDcount] );
    if (CommError AND E_FIRSTCHAR)>0 then
      ErrorListBox1.Items.Add('Error Type = ' + 'FirstChar [' + Format('%x',[Integer(FirstChar)]) + ']');
    if (CommError AND E_SECONDCHAR)>0 then
      ErrorListBox1.Items.Add('Error Type = ' + 'SecondChar [' + Format('%x',[Integer(SecondChar)]) + ']');
    if (CommError AND E_CHARMATCH)>0 then
      ErrorListBox1.Items.Add('Error Type = ' + 'CharMatch[' + Format('%x',[Integer(SecondChar)]) + ']');
    if (CommError AND E_BUFFERMAX)>0 then begin
      ErrorListBox1.Items.Add('Error Type = ' + 'BufferMax');
      StopComm; end;
    CommError := 0;
    FirstChar := Char(0);
    SecondChar := Char(0);
    end;
  If RxError > LastRxError then begin
    fmDebug.AddDebugItem( '**RxError: ' );
    LastRxError := RxError;
    end;

end;

//************************ ErrorMaskBoxChange *******************************
procedure TfmComm.ErrorMaskBoxChange(Sender: TObject);
begin
  try
    ErrorMask := StrToInt( '$'+fmComm.ErrorMaskBox.Text );
  except
    ErrorMask := $ff;
  end;
  fmComm.ErrorMaskBox.Text := Format( '%x', [ErrorMask] );
end;

//************************ Forms *******************************
procedure TfmComm.FormCreate(Sender: TObject);
begin
  fmDebug.AddDebugItem( 'FormCreate Comm ' );
  ErrorMask := $ff;
  ElapsedTime := 0;
  RxError := 0;
  MaxRxDcount := 0;
  CommCriticalSection := TCriticalSection.Create;
  CommSerialPort := TSerialPortNG.Create(nil);
  CommOK := False;
  Sampling := False;
  fmComm.Left := (Screen.Width div 2) + (fmComm.Width div 2) + 2;
  CommSerialPort.OnRxClusterEvent := fmComm.ReadCluster;
  CommSerialPort.OnProcessError := fmComm.ProcessError;
  fmDebug.AddDebugItem( 'End FormCreate Comm ' );
end;

procedure TfmComm.FormDestroy(Sender: TObject);
begin
CommTimer.Enabled := False;
if CommSerialPort <> nil then begin
  CommSerialPort.Free;
  CommSerialPort := nil;
  end;
Delay(100); // This suppresses an AV error
if CommCriticalSection <> nil then begin
  CommCriticalSection.Free;
  CommCriticalSection := nil;
  end;
end;

procedure TfmComm.FormShow(Sender: TObject);
begin
  ErrorMaskBox.Text := Format( '%x', [ErrorMask] );
  fmComm.Left := Screen.Width - fmComm.Width;
end;

//************************ GetCommPortName *******************************
function TfmComm.GetCommPortName: String;
var      TestPortName: String;
         SN, DESC: PChar;
         i: Integer;
         PortList: TStrings;
begin
    fmDebug.AddDebugItem( 'Getting CommPortName ' );
    If CommSerialPort.Active then begin
      Result := CommSerialPort.CommPort;
      exit; end;
    TestedPort := 'NONE';
//    CommSerialPort.Active := False;
    SerialPortOff;
    CommOK := False;
    PortList := TStringList.Create;
    GetPortList(PortList);                   //Fill the list
    for i := 1 to PortList.Count do
      fmDebug.AddDebugItem( Trim(PortList.Strings[i-1]) );

    SN := DEVSN1;
    DESC := DEVDESC1;
    CommPortInfo := GetCommPortInfo(DEVID,SN);
    If StrLComp( pchar(CommPortInfo.FriendlyName), DESC, strlen(DESC) )=0 then begin //1
      fmDebug.AddDebugItem( 'FriendlyName: ' + CommPortInfo.FriendlyName );
      TestPortName := CommPortInfo.PortName;
      fmDebug.AddDebugItem( 'Opening ' + TestPortName + ' ' );
      if fmComm.OpenComm(TestPortName) then begin //2
        fmDebug.AddDebugItem( 'Testing ' + TestPortName + ' ' );
        fmComm.ClearInput(True);
        fmComm.SendStr( chr(Ctrl_T) );
        Delay(400);
        if fmComm.ReadTestString then begin //3
          TestedPort := TestPortName;
          CommOK := True;
          fmComm.ClearInput(True);
          CommSerialPort.Active := True; end; //-3
        end; //-2
      end; //-1
    If not CommOK then begin //1
      SN := DEVSN2;
      DESC := DEVDESC2;
      CommPortInfo := GetCommPortInfo(DEVID,SN);
      If StrLComp( pchar(CommPortInfo.FriendlyName), DESC, strlen(DESC) )=0 then begin //2
        fmDebug.AddDebugItem( 'FriendlyName: ' + CommPortInfo.FriendlyName );
        TestPortName := CommPortInfo.PortName;
//        SerialPortOff;
        fmDebug.AddDebugItem( 'Opening ' + TestPortName + ' ' );
        if fmComm.OpenComm(TestPortName) then begin //3
          fmDebug.AddDebugItem( 'Testing ' + TestPortName + ' ' );
          fmComm.ClearInput(True);
          fmComm.SendStr( chr(Ctrl_T) );
          Delay(400);
          if fmComm.ReadTestString then begin //4
            TestedPort := TestPortName;
            CommOK := True;
            fmComm.ClearInput(True);
            CommSerialPort.Active := True; end; //-4
          end; //-3
        end; //-2
      end; //-1
    PortList.Free;
    fmDebug.AddDebugItem( 'Tested Port: '+ TestedPort );
    Result := TestedPort;
end;

//************************ HideButton *******************************
procedure TfmComm.HideButtonClick(Sender: TObject);
begin
  fmComm.Visible := False;
end;

//************************ OpenComm *******************************
function TfmComm.OpenComm( Name: String ): Boolean;
begin
  fmDebug.AddDebugItem( 'OpenComm ' + Name );
  if CommSerialPort <> nil then begin
    CommSerialPort.CommPort := Name;
    CommSerialPort.BaudRate := CBR_57600;
    CommSerialPort.ParityType := NOPARITY;
    CommSerialPort.StopBits := ONESTOPBIT;
    CommSerialPort.DataBits := 8;
    CommSerialPort.FlowControl := fcNone;
    CommSerialPort.StripNullChars := False;
    CommSerialPort.XTOAuto := True;
    CommSerialPort.Active := True;
    CommSerialPort.RxQueueSize := RXQUEUE; // 1 second of characters
    if CommSerialPort.Active then begin
      CommSerialPort.ClearTxDQueue;
      CommSerialPort.ClusterSize := 2;
      Result := True; end
    else
      Result := False;
  end
  else
    Result := False;
end;

//************************ ProcessError *******************************
procedure TfmComm.ProcessError(Sender: TObject; Place,
  Code: Cardinal; Msg: String; Noise: Byte);
begin
  fmComm.ErrorListBox1.Items.Add(Msg);
end;

//************************ ReadBuffer *******************************
// ReadBuffer converts two bytes to an integer representing current data
// The CommRead pointer and RxDataCount increments and the CommCharCount decrements
// Actual data 0-4095 is offset by 2048 for bipolar current
// Returns signed integer data

function TfmComm.ReadBuffer : Integer;
var
  ch1, ch2: Char;
  RecdData: Word;
  DataChk: SmallInt;

begin
     RecdData := 2048;  //Default for zero
     CommCriticalSection.Enter;
     if CommCharCount >= 2 then begin
       ch1 := CommBuffer[ CommReadPtr ];  // Read first character
       dec( CommCharCount );
       inc( CommReadPtr );
       if CommReadPtr > MAXCOMM then CommReadPtr := 0;
       if (Byte(ch1) and $01 <> 0) then begin
         // Wrong sequence, exit and read next byte
         Inc(RxError);                    // Bit 0 of LSB must be 0
         CommError := CommError or E_FIRSTCHAR; //eFirstChar;
//         fmComm.Visible := True;
         result := 0;
         CommCriticalSection.Leave;
         If CommError > 0 then
           FirstChar := ch1;
         exit;
         end;

       ch2 := CommBuffer[ CommReadPtr ];  // Read second character
       dec( CommCharCount );
       inc( CommReadPtr );
       if CommReadPtr > MAXCOMM then CommReadPtr := 0;
       if (Byte(ch2) and $01 <> 1) then begin
         // Wrong sequence, exit and read next byte
         Inc(RxError);                    // Bit 0 of LSB must be 0
         CommError := CommError or E_SECONDCHAR;  //eSecondChar;
//         fmComm.Visible := True;
         result := 0;
         CommCriticalSection.Leave;
         If CommError > 0 then
           SecondChar := ch2;
         exit;
         end;

       DataChk := BYTE(ch1) AND $02;
       If (BYTE(ch2) AND $02) <> DataChk then begin
         Inc(RxError);                        // Two low bits must be equal
         CommError := CommError or E_CHARMATCH;  //eCharMatch;
//         fmComm.Visible := True;
         Result := 0;
         end
       else begin
         RecdData := BYTE(ch2) AND $FC;      // MSB
         RecdData := RecdData shl 6;
         RecdData := RecdData OR BYTE(ch1);  // LSB
         RecdData := RecdData AND $0FFC;     // 10 bit precision
         Result := Integer(RecdData) - 2048;   // Actual signed data
         end;
       end
     else // If CommCharCount < 2 (ERROR)
       Result := 0;                          // Default if error
  CommCriticalSection.Leave;
end;

//************************ ReadTestString (CommTest) **************************
function TfmComm.ReadTestString: Boolean;
begin
  if (fmComm.CommCharCount > 200) and (fmComm.CommBuffer[5] = '1') then
    Result := True
  else begin
    Result := False;
    end;
end;

//************************ ReadCluster *******************************
procedure TfmComm.ReadCluster;
var
          RecdPtr: PChar;   //^char;
          CSize, NCSize : Integer;
          n: Integer;
          RdErrors: DWord;
          ErrorCode: Integer;
begin
  CSize := CommSerialPort.ClusterSize;
  NCSize := CommSerialPort.NextClusterSize;
  if NCSize >= 0 then
    begin
      ErrorCode := CommSerialPort.NextClusterCCError;
      if ErrorCode <> 0
        then inc(RxError);

      CommCriticalSection.Enter;
      RecdPtr := CommSerialPort.ReadNextCluster( NCSize, RdErrors );
      RxDcount := NCSize;
      if RxDcount > MaxRxDcount then
        MaxRxDcount := RxDcount;
      if (RxDcount >= (CommSerialPort.RxQueueSize - CSize) ) then begin //2
        inc( rxError );
        CommError := CommError or E_BUFFERMAX;  //eBufferMax;
        end; //2
      For n:= 1 to RxDcount do begin //2
        fmComm.CommBuffer[fmComm.CommBufferPtr] := RecdPtr[n-1];  //RecdPtr^
 //       inc(RecdPtr);
        inc(fmComm.CommBufferPtr);
        if(fmComm.CommBufferPtr > MAXCOMM) then
          fmComm.CommBufferPtr := 0;
        inc(fmComm.CommCharCount);
        if(fmComm.CommCharCount > MAXCOMM) then
          fmComm.CommCharCount := MAXCOMM+1;
      end; //2
      FreeMem( RecdPtr, NCSize );         // Added 4/8/09
      CommCriticalSection.Leave;

    end;
end;

//************************ SendChar *******************************
procedure TfmComm.SendChar( ch: char );
begin
  fmComm.SendStr( ch );
end;

//************************ SendRange *******************************
procedure TfmComm.SendRange( RangeNum: Integer );
var RngChar: Char;
begin
  RngChar := Char( (8-RangeNum) + 48);  //Used to send range number to PIC
  fmDebug.AddDebugItem( 'Send Range '+ RngChar );
  if CommSerialPort.Active then begin
    if Sampling then
      fmComm.StopComm;
    fmComm.SendChar(RngChar);
    fmComm.ClearInput(True);
    fmComm.StartComm;
    end;
end;

//************************ SendStr *******************************
procedure TfmComm.SendStr( S: String );
var       Timeout: Integer;
begin
  Timeout := 10;
  While Timeout > 0 do begin
    if CommSerialPort.SendInProgress then begin
      fmDebug.AddDebugItem( 'SendStr Delay ' );
      Delay(100);
      Dec( Timeout );
      end
    else
      break;
  end;
  CommSerialPort.SendString ( S );
end;

//************************ SerialPortOff *******************************
procedure TfmComm.SerialPortOFF();
begin
  fmDebug.AddDebugItem( 'SerialPortOFF ' );
  If Sampling then
    StopComm;
  ClearInput(True);
  CommSerialPort.Active := False;
end;

//************************ SerialPortOn *******************************
// Set up comm port parameters and activate port name specified by ComPort

function TfmComm.SerialPortON(ComPort: String): Boolean;
begin
  fmDebug.AddDebugItem( 'SerialPortON ' );
  if OpenComm( ComPort ) then begin
    Result := True; end
  else
    Result := False;
end;

//************************ StartComm *******************************
procedure TfmComm.StartComm;
begin
  fmDebug.AddDebugItem( 'StartComm ' );
  if fmComm.CommOK then begin  //(TestComm( ComPort ) = sOK)
    DelayMsec(100);                         //Added 2/12/10
    ClearInput(True);
    SendChar(Chr(Ctrl_A));        // Start transmission
    Sampling := True;
    RxDerrorCount := 0;
  end;
end;

//************************ StopComm *******************************
procedure TfmComm.StopComm;
begin
  fmDebug.AddDebugItem( 'StopComm ' );
  SendChar(Chr(Ctrl_D));        // Stop transmission
  Sampling := False;
end;

//************************ TestComm *******************************
function    TfmComm.TestComm( name: String): TCommPortStatus;
var         S: String;
            TestChar: Char;
            Timeout: Integer;
begin
  fmDebug.AddDebugItem( 'TestComm ' + name );
  Result := sNone;

  Timeout := 10;
  S := 'A' + CR;
  if not CommSerialPort.Active then
    OpenComm( name );
  if not CommSerialPort.Active then
    Result := sOpenError
  else begin //1
    StopComm;
    ClearInput(True);
    fmDebug.AddDebugItem( 'Sending CR ' );
    SendChar(char(CR));
    DelayMsec(100);
    fmDebug.AddDebugItem( 'CommCharCount='+IntToStr(CommCharCount) );
    fmDebug.AddDebugItem( 'Clearing Input ' );
    ClearInput(True);
    DelayMsec(100);
    fmComm.SendStr( S );
    fmDebug.AddDebugItem( 'Sent test string ' );
    While Timeout > 0 do begin //2
      If CommCharCount > 0 then
        break
      else begin //3
        Delay(100);
        dec(Timeout);
      end; //-3
    end; //-2
    If CommCharCount > 0 then begin //2
      fmDebug.AddDebugItem( 'Received test characters ' );
      TestChar := CommBuffer[CommBufferPtr-2];
      If TestChar = 'A' then begin //3
        Result := sOK;
        CommOK := True;
        end //-3
      else begin //3
        fmDebug.AddDebugItem( 'Wrong test char '+ TestChar );
        Result := sCharError;
        CommOK := False;
        end; //-3
      end //-2
    else begin //2
      fmDebug.AddDebugItem( 'Comm Test Timeout' );
      Result := sTimeout;
      CommOK := False;
      end; //-2
    end; //-1
  fmDebug.AddDebugItem( 'End TestComm ' );
end;

// Send calibration data to PIC memory in text file format with CRLFs
// Data is stored in string S and sent as a single string
// CTRL_D makes sure data transmission has stopped
// CTRL_E signals PIC to receive 128 bytes of data into PIC Program Flash Memory
// CTRL_D at end signals end of transmission

procedure TfmComm.SendCal( Data: Array of Char );
var       S: String;
begin
  fmDebug.AddDebugItem( 'SendCal ' );
  If CommSerialPort.Active then begin
    S := char(CTRL_D) + char(CTRL_E);       // Start with two control characters
    S := S + Data;
    S := S + char(CTRL_D);
    // Total string length should be 2+128+1 = 131
    if( MessageDlg('Writing to PIC Memory',mtWarning,[mbOK,mbCancel], 0)) = mrOK
      then begin
      SendStr( S );
      Delay(300);                // Wait for transmission to complete
      end;
    end;
end;

// StartCal sends character to instruct PIC to send cal data string on serial port

//procedure TfmComm.StartCal( Var Data: String );
//begin
//end;

// ReceiveCal tries up to 10 100mSec timer ticks to receive entire string of 128 bytes
// If all bytes are received in CommBuffer the string is checked for error
// If array element 126 is a CR the string is OK and is written to the CalFile
// Otherwise CalError is set and Default Cal is used

function TfmComm.ReceiveCal( Var sData: Array of char ): Boolean;
var       ch1: char;
          i: Integer;
          s: Array[0..127] of char;
          CalTimerCount: Integer;
begin
  fmDebug.AddDebugItem( 'ReceiveCal ' );
  Result := False;
  If not CommSerialPort.Active then
    exit;
  StopComm;
  ClearInput(True);
  Delay(100);
  fmComm.SendChar( char(CTRL_C) );
  Delay(100);                   // Seems to be needed 3/14/09
  CalTimerCount := 10;
  While CalTimerCount > 0 do begin //1
    if fmComm.CommCharCount >= 128 then begin //2 All data has been received
      CalTimerCount := 0; // Done
      ch1 := fmComm.CommBuffer[ 8 ];
      If ch1 = char(13) then begin
        Result := True;
        // Write PIC data to array
        for i := 0 to 127 do
          s[i] := fmComm.CommBuffer[i];
        StrLCopy(sData, s, 128);
        end; //3
      end //2
    else begin //2 Waiting for data
      fmDebug.AddDebugItem( 'Waiting for Cal Data ' );
      dec( CalTimerCount );
      if CalTimerCount > 0 then
        Delay(100);
      end; //2
    end; //1
  fmDebug.AddDebugItem( 'End ReceiveCal ' );
end;


procedure TfmComm.BtnPortsClick(Sender: TObject);
var       S: String;
          i: Integer;
          FriendlyName: String;
          ComPortName: String;
          Reg: TRegistry;
          RegInfo: TRegKeyInfo;
          RegValueNames: TStringList;
          RegKeyNames: TStringList;
          PortList: TStrings;
begin
  Reg := TRegistry.Create;
  if Reg = nil then
    fmDebug.AddDebugItem( 'Error creating Reg ' );
  RegValueNames := TStringList.Create;
  RegKeyNames := TStringList.Create;
  Reg.RootKey := HKEY_LOCAL_MACHINE;
  S := 'SYSTEM\CurrentControlSet\Enum\USB';
  S := S + '\Vid_04d8&Pid_fdcf\0123456789';
  fmDebug.AddDebugItem( 'Key: ' + S );
  If Reg.KeyExists(S) then begin //1
    Reg.OpenKeyReadOnly(S);
    Reg.GetKeyInfo(RegInfo);
    Reg.GetKeyNames(RegKeyNames);
    ErrorListBox1.Items := RegKeyNames;
      ErrorListBox1.Items.Append('***Value Names***');
    Reg.GetValueNames(RegValueNames);
    ErrorListBox1.Items.AddStrings( RegValueNames );
    FriendlyName := Reg.ReadString('FriendlyName');
    ErrorListBox1.Items.Append(FriendlyName);
    S := 'Device Parameters';
    If Reg.KeyExists(S) then begin //2
      Reg.OpenKeyReadOnly(S);
      ComPortName := Reg.ReadString('PortName');
      ErrorListBox1.Items.Append(ComPortName); end; //-2
    end //-1
  else
    fmDebug.AddDebugItem( 'Key does not exist ' );
  RegKeyNames.Free;
  RegValueNames.Free;
  Reg.Free;

  PortList := TStringList.Create;
  GetPortList(PortList);                   //Fill the list
  for i := 1 to PortList.Count do
    ErrorListBox1.Items.Append(Trim(PortList.Strings[i-1]));
  PortList.Free;

end;

function   GetCommPortInfo( id: String; sn: String ): TCommPortInfo;
var        S: String;
           Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  if Reg = nil then
    fmDebug.AddDebugItem( 'Error creating Reg ' );
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    S := 'SYSTEM\CurrentControlSet\Enum\USB';
    S := S + '\' + id + '\' + sn;           // Vid_04d8&Pid_fdcf\0123456789';
    fmDebug.AddDebugItem( 'Key: ' + S );
    try
      Reg.OpenKeyReadOnly(S);
      result.FriendlyName := Reg.ReadString('FriendlyName');
      S := 'Device Parameters';
      Reg.OpenKeyReadOnly(S);
      result.PortName := Reg.ReadString('PortName');
    except
      result.FriendlyName := '';
      result.PortName := '';
    end;
  finally
    Reg.Free;
  end;
end;

{Create File Association - Example code only}
procedure CreateFileAssociation;
 var reg: TRegistry;
 begin
 reg := TRegistry.Create;
 reg.RootKey := HKEY_CLASSES_ROOT;
 reg.LazyWrite := false;
 {Add Program Support}
 reg.OpenKey('.bor\shell\open\command', true);
 {Invoke the program passing the file name as the first parameter}
 reg.WriteString('', 'C:\Program Files\Borland\Delphi 3\Project .exe % ');
 {Add Icon Display} reg.CloseKey; reg.OpenKey('.bor\DefaultIcon', true);
 {Use the first icon in the executable to display}
 reg.WriteString('', 'C:\Program Files\Borland\Delphi 3\Project .exe,0');
 reg.CloseKey;
 reg.free;
 end;

{ Testing results - 2/4/06

Ran overnight with USB Port COM3 (in Delphi IDE)
23750 seconds, 406 errors, => 3.56 errors/million

Ran about 800 seconds with native port COM1 => 0.5 errors/million

Timing test: 8500 seconds, 56 errors
Stopwatch: 2:21:30 = 8490 seconds = 0.12% error
Error Rate: 1.37/million

New test 8/2/06 on Win XP machine
44,445 seconds (12.2 hrs), 0 errors

// 5/11/05 - Stop Bits was 2
// 5/12/05 - Mask error bit
// 11/30/05 - New Event Scheduler, add sampling variable
// 12/16/05 - Remove PortActive variable, add comments
// 1/21/06 - Fixed comm check routine, delay on exit
//           Still problem in scheduler -events not consecutive
// 2/1/06 - Now uses Delay for all Comm functions. Added timeout on Send
//          Timer interval now 110 for closer tolerance (55 mSec granularity)
//          Added MaxRxDcount, Error detect overrun, Comm Queue now 32000
// 2/3/06 - Change receive data check
// 2/6/06 - Incorporated DLL code here
// 8/1/06 - Changed all SerialPortNG1 to OrtmasterSerialPort
// 10/1/06 - Goal: eliminate Access Error on first run
//           Allow continuous current to run after test complete
//           Redo as OOP
// 02/03/07 - Moved cal send/receive from OrtCal
// 03/03/09 - Added Date, shortened Serial Number, simplified ReceiveCal
// 3/14/09 - Rewritten to be modular and reusable for similar hardware and software
// 3/18/09 - Removed CommTest object and unused items, changed Public/Private
// 04/08/09 - Fixed major bug in ReadCluster
// 04/14/09 - Changed to allow ORTM-4 and BTSA-3 hardware
// 06/21/09 - Added ShowDebug to ClearComm, Add RXQUEUE
// 02/12/10 - Added return from GetCommPortName if Active, added FirstChar etc. Debug
}

end.


