unit Setup; // Create this 5th, after Debug, Comm, Cal, and Wave

// Cleaned up 3/14/09
// 12/22/09 - Made universal; must set SetupFilename and DataFilename
// SourceName added. Set to USB, COM, or WAV

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Debug;

type
  TTestMode = (NORMAL,MINPU);

type
  TfmSetup = class(TForm)
    ebShuntAmperes: TEdit;
    LabelShuntAmperes: TLabel;
    ebShuntMillivolts: TEdit;
    LabelShuntMillivolts: TLabel;
    Button1: TButton;
    lblMaxOnTime: TLabel;
    ebMaxOnTime: TEdit;
    lblMaxOffTime: TLabel;
    ebMaxOffTime: TEdit;
    lblDataFileName: TLabel;
    ebDataFileName: TEdit;
    lblSourceName: TLabel;
    lblMaxOps: TLabel;
    ebMaxOps: TEdit;
    cbSourceName: TComboBox;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Button1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    DataFileName: String;
    MaxOffTime: Real;
    MaxOnTime: Real;
    NormalMaxOps: Integer;
    RangeNum: SmallInt; // Varies from 1 to 8, Stored as 1..8 in setup file (10/10/08)
    SetupFileName: String;   // = SETUPFILE
    ShuntAmps: Real;         // = 1000.0;
    ShuntMillivolts: Real;   // = 100.0;
    SourceName: String;      // USB, COMnn, or Waveform Filename
    Source: String;          // USB, COM, or WAV
    TestMode: TTestMode;
    procedure ReadSetupFile;
    procedure WriteSetupFile;
    { Public declarations }
  end;

var
  fmSetup: TfmSetup;

implementation

{$R *.DFM}

procedure TfmSetup.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  fmDebug.AddDebugItem( 'Setup Form Close ' );
  Action := caHide;
  MaxOnTime := StrToFloat( ebMaxOnTime.Text );
  MaxOffTime := StrToFloat( ebMaxOffTime.Text );
  NormalMaxOps := StrToInt( ebMaxOps.Text );
  DataFileName := ebDataFileName.Text;
  SourceName := cbSourceName.Text;
  ShuntAmps := StrToFloat( ebShuntAmperes.Text );
  ShuntMillivolts := StrToFloat( ebShuntMillivolts.Text );
  WriteSetupFile;
  ReadSetupFile;
  ModalResult := mrOK;
end;

procedure TfmSetup.Button1Click(Sender: TObject);
begin
  fmSetup.Close;
end;

procedure TfmSetup.FormShow(Sender: TObject);
begin
  fmDebug.AddDebugItem( 'Setup Form Show ' );
  fmSetup.Left := Screen.Width - fmSetup.Width;
  ebMaxOnTime.Text := Format( '%4.2f', [MaxOnTime] );
  ebMaxOffTime.Text := Format( '%4.2f', [MaxOffTime] );
  ebMaxOps.Text := Format( '%d', [NormalMaxOps] );
  ebDataFileName.Text := DataFileName;
  cbSourceName.Text := SourceName;
  ebShuntAmperes.Text := Format( '%5.1f', [ShuntAmps] );
  ebShuntMillivolts.Text := Format( '%3.2f', [ShuntMillivolts] );
end;

// Setup File format
// RangeNum: Int [1..8]    Was 0..7
// MaxOnTime: Float [0.0..50.0]
// MaxOffTime: Float [0.0..50.0]
// DataFileName: String
// SourceName: String [USB,COM1..COM8] Demo Mode if cannot be opened
// MaxOps: Int [0..5] 0 indicates minimum pickup
// TestMode: NORMAL, MINPU
// ShuntAmperes: 100, 200, 2500, 5000, etc.
// ShuntMillivolts: 50, 100, etc.

procedure TfmSetup.ReadSetupFile;
var
  setup_file:  TextFile;
  S: string;
  i: Integer;

  begin

  fmDebug.AddDebugItem( 'ReadSetupFile ' );
  // First check if file exists; if not create with default values
  if not FileExists( SetupFileName ) then begin
    // Assign default values
    Application.MessageBox( 'Creating Default Setup File',
      'File Missing', MB_OK );
    RangeNum := 4;
    MaxOnTime := 5.000;
    MaxOffTime := 5.000;
    if DataFileName = '' then
      DataFileName := 'Ortmaster.odf';
    NormalMaxOps := 4;
    TestMode := NORMAL;
    ShuntAmps := 1000;
    ShuntMillivolts := 100.0;
    WriteSetupFile;
    end;

  If FileExists( SetupFileName ) then begin
    AssignFile( setup_file, SetupFileName );
    reset( setup_file);
    readln(setup_file, S );
    RangeNum := StrToInt( Trim(S) ); // Stored as 1..8 in setup file   (10/10/08)
    If( (RangeNum < 1) or (RangeNum > 8) ) then
      RangeNum := 4;                 // Added check 10/10/08
    readln(setup_file, S );
    MaxOnTime := StrToFloat( Trim(S) );
    readln(setup_file, S );
    MaxOffTime := StrToFloat( Trim(S) );
    readln(setup_file, S );
    DataFileName := Trim(S);
    readln(setup_file, S );
    SourceName := Trim(S);
    readln(setup_file, S );
    NormalMaxOps := StrToInt( Trim(S) );
    readln(setup_file, S);
    TestMode := TTestMode(StrToInt(Trim(S)));
    readln(setup_file, S);
    ShuntAmps := StrToFloat( Trim(S) );
    readln(setup_file, S);
    ShuntMillivolts := StrToFloat( Trim(S) );
    closefile(setup_file);
  end;
  if trim(SourceName) = 'USB' then
    Source := 'USB'
  else if StrLIcomp( PChar(SourceName), 'COM', 3 ) = 0 then
    Source := 'COM';
//  else
//    Source := 'WAV';
end;

procedure TfmSetup.WriteSetupFile;
var
  setup_file:  TextFile;
  S: String;

  begin

  fmDebug.AddDebugItem( 'WriteSetupFile ' );
//  FileName := SETUPFILE;
  try
    AssignFile( setup_file, SetupFileName );
    rewrite( setup_file);
    writeln(setup_file, Format( '%d', [RangeNum] ) ); // Stored as 1..8 in setup file
    writeln(setup_file, Format( '%4.2f', [MaxOnTime] ) );
    writeln(setup_file, Format( '%4.2f', [MaxOffTime] ) );
    writeln(setup_file, DataFileName);
    writeln(setup_file, SourceName);
    writeln(setup_file, Format( '%d', [NormalMaxOps] ) );
    S := Format( '%d', [Integer(TestMode)] );
    writeln(setup_file, S);
    writeln(setup_file, Format( '%6.0f', [ShuntAmps] ) );
    writeln(setup_file, Format( '%6.0f', [ShuntMillivolts] ) );
    closefile(setup_file);
  except
     MessageDlg('Error writing file: '+SetupFileName, mtError, [mbAbort], 0);
   end;
end;

procedure TfmSetup.FormCreate(Sender: TObject);
begin
  fmDebug.AddDebugItem( 'FormCreate Setup ' );
  fmSetup.Left := (Screen.Width) - (fmSetup.Width) - 2;
end;

end.
