unit USB;

// 12/8/09 - Added to Ortmaster project to replace COM port functions
// Try to connect and check to see if connected by doing a device count on the pid_vid
// Start Comm by sending Ctrl-A, Stop Comm by sending Ctrl-D
// 12/22/09 - Removed dependencies on specific files
// 01/05/10 - Make functions into threads
// 01/11/10 - Clean up and remove thread functions, add delays as required

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  syncobjs, Delays, Registry,
  Debug,
  StdCtrls, ExtCtrls;

Const
  MAXSIZE=64;
  MAXBUFF=65000;
  MPUSB_FAIL=0;
  MPUSB_SUCCESS=1;
  MP_WRITE:DWORD=0;
  MP_READ:DWORD=1;
  MAX_NUM_MPUSB_DEV=127;
  READ_VERSION    = $00;
  RESET_DEVICE    = $FF;
  ORTM4_VID_PID   = 'vid_04d8&pid_fdcf';
  TMO_VAL = 25;

  E_FIRSTCHAR = $01;
  E_SECONDCHAR = $02;
  E_CHARMATCH = $04;
  E_BUFFERMAX = $08;
  Ctrl_A = $01;  //Start Transmission
  Ctrl_C = $03;  //Return Calibration Data in Flash
  Ctrl_D = $04;  //Stop Transmission
  Ctrl_E = $05;  //Sending Calibration Data to Flash
  Ctrl_T = $14;  //Test
  CR = $0d;
  LF = $0a;
  DEVID = 'Vid_04d8&Pid_fdcf';
  DEVSN1 = '0123456789';
  DEVSN2 = 'ETI20090414';
  DEVSN3 = 'PST0123456';
  DEVDESC1 = 'Ortmaster';
  DEVDESC2 = 'BTSA-3';

type
  DWORD = LongInt;
  PCHAR8 =  array[0..MAXSIZE+3] of char;
  PBYTE = array[0..MAXSIZE+3] of BYTE;
  PDWORD =  array[0..MAXSIZE+3] of DWORD;
  PVOID = Pointer;
  UINT = Cardinal;

type  TUSBPortInfo = record
        ClassName: String;
        Service: String;
        end;

type  TUSBStatus = (uNone, uNotConnected, uOpenError, uTimeout, uCharError, uOK );

type  TcommBuff = record
  inptr:         word;
  outptr:        word;
  count:         word;
  buff:          array[0..MAXBUFF] of BYTE;
end;

type
  TfmUSB = class(TForm)
    Memo1: TMemo;
    ComboBox1: TComboBox;
    ebUSBError: TEdit;
    Label1: TLabel;
    Edit2: TEdit;
    USBTimer: TTimer;
    Label2: TLabel;
    ebUSBRxData: TEdit;
    Label3: TLabel;
    ebUSBMaxRxData: TEdit;
    Label4: TLabel;
    procedure USBTimerTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    ElapsedTime: LongWord;
    ErrorMask: Integer;
    LastRxError: Integer;
    MyPriority: TThreadPriority;

    USBCommError: Byte;           // CommErrorType = eNone;
    function  WriteBuffer(uchar: BYTE; var buffer: tCommBuff): Boolean;
  public
    Tmo: Integer;
    USBCommOK: Boolean;      //Determined at USBTestComm or CloseUSB
    USBMaxRxdCount: Integer;
    USBRxDCount: Integer;
    USBRxDerrorCount: Integer;
    USBRxError: Integer;
    USBSampling: Boolean;     //Set at StartSampling, reset at StopSampling
    USBCommBuffer: TcommBuff;
    USBThreadRunning: Boolean;

    procedure InitBuffer(var buffer: tCommBuff);
    function  OpenUSB: Boolean;
    procedure CloseUSB;
    function  GetUSBPortInfo( id: String; sn: String ): TUSBPortInfo;
    function  ReadBuffer(var buffer: tCommBuff): BYTE;
    function  USBConnected(pid_vid: PCHAR8): Boolean;
    function  USBReadBuffer : Integer;
    function  USBReceiveCal( Var sData: Array of char ): Boolean;
    procedure USBSendCal( Data: Array of char );
    procedure USBSendRange( RangeNum: Integer );
    procedure USBStartSampling;
    procedure USBStopSampling;
    function  USBSendChar( ch: char ): Boolean;
    function  USBSendStr( S: String ): Boolean;
    function  USBTestComm: TUSBStatus;

  end;

// This thread runs continuously and reads the USB port into the CommBuffer
type
  TUSBThread = class(TThread)
  private
    procedure UpdateMemo;
    procedure SetNil(Sender: TObject);
    { Private declarations }
  protected
    constructor Create(CreateSuspended: Boolean);
    procedure Execute; override;
  end;

var
  vid_pid:PCHAR8=ORTM4_VID_PID;
  out_pipe:PCHAR8='\MCHP_EP1';         //char out_pipe[]= "\\MCHP_EP1";
  in_pipe:PCHAR8='\MCHP_EP1';          //char in_pipe[]= "\\MCHP_EP1";
  myOutPipe:THANDLE = INVALID_HANDLE_VALUE;
  myInPipe:THANDLE = INVALID_HANDLE_VALUE;
  selection:DWORD;     // Used in _MPUSBOpen
  send_buf:PBYTE;      //  PBYTE = array[0..MAXSIZE] of BYTE;
  receive_buf:PBYTE;
  RecvLength:DWORD;
  ThreadStr: String;
  DataCount, LoopCount: Integer;
  TotalCount: longint;
  TotalData: LongInt = 0;

  USBCommCriticalSection: TCriticalSection;
  thUSBread: TUSBThread;

var
  fmUSB: TfmUSB;

function SendReceivePacket(SendData:PBYTE; SendLength:DWORD; var ReceiveData:PBYTE;
  var ReceiveLength:DWORD; SendDelay:UINT; ReceiveDelay:UINT):DWORD; stdcall;

function _MPUSBGetDLLVersion():DWORD;
             cdecl;external 'mpusbapi.dll';
function _MPUSBGetDeviceCount(pVID_PID:PCHAR8):DWORD; 
             cdecl;external 'mpusbapi.dll';
function _MPUSBOpen(instance:DWORD;pVID_PID:PCHAR8;
             pEP:PCHAR8;dwDir:DWORD;dwReserved:DWORD):
             THANDLE;cdecl;external 'mpusbapi.dll';
function _MPUSBClose(handle:THANDLE):DWORD; 
             cdecl;external 'mpusbapi.dll';
function _MPUSBRead(handle:THANDLE;var pData:PBYTE;
             dwLen:DWORD;var pLength:DWORD;
             dwMilliseconds:DWORD):DWORD;cdecl;
             external 'mpusbapi.dll';
function _MPUSBReadInt(handle:THANDLE;
             var pData:PBYTE;dwLen:DWORD;
             var pLength:PDWORD;
             dwMilliseconds:DWORD):DWORD;
             cdecl;external 'mpusbapi.dll';
function _MPUSBWrite(handle:THANDLE;pData:PBYTE;
             dwLen:DWORD;
             var pLength:DWORD;
             dwMilliseconds:DWORD):DWORD; 
             cdecl;external 'mpusbapi.dll';


implementation

{$R *.DFM}

var  fThreadTerminate: TNotifyEvent;

procedure CheckInvalidHandle();
begin
  if(GetLastError=ERROR_INVALID_HANDLE) then
  begin
    if myOutPipe = INVALID_HANDLE_VALUE then
        _MPUSBClose(myOutPipe);
    if myInPipe = INVALID_HANDLE_VALUE then
        _MPUSBClose(myInPipe);
    myInPipe:=INVALID_HANDLE_VALUE;
    myOutPipe:=INVALID_HANDLE_VALUE;
  end
  else
    ShowMessage('Error Code :'+inttostr(GetLastError()));
end;

function SendReceivePacket(SendData:PBYTE; SendLength:DWORD; var ReceiveData:PBYTE;
var ReceiveLength:DWORD; SendDelay:UINT; ReceiveDelay:UINT):DWORD; stdcall;
var
  SentDataLength:DWORD ;
  ExpectedReceiveLength:DWORD;
begin
 ExpectedReceiveLength:= ReceiveLength;
 Result:=0; // default to Failed
 if((myOutPipe <> INVALID_HANDLE_VALUE) and (myInPipe <> INVALID_HANDLE_VALUE)) then
   begin
    if(_MPUSBWrite(myOutPipe,SendData,SendLength,SentDataLength,SendDelay)<>0) then
     if(_MPUSBRead(myInPipe,ReceiveData,ExpectedReceiveLength,ReceiveLength,ReceiveDelay)<>0) then
       begin
       if(ReceiveLength = ExpectedReceiveLength) then
        begin
         Result:=1; // Success
         exit;
        end
       else
        if(ReceiveLength < ExpectedReceiveLength) then
         begin
           Result:=2; // incorrect receive length
           exit;
         end
        end
     else
      CheckInvalidHandle()
    else
      CheckInvalidHandle()
   end
 else
   begin
     Result:=0; // Failed
   end;
end;

//======================== USBThread procedures ================================
constructor  TUSBThread.Create(CreateSuspended: Boolean);
begin
     inherited Create(CreateSuspended);
     FreeOnTerminate := True;
end;

procedure TUSBthread.UpdateMemo;
begin
  if fmUSB.Memo1 <> nil then
    fmUSB.Memo1.Lines.Add(ThreadStr);
end;

procedure TUSBThread.SetNil(Sender: TObject);
begin
  thUSBread := nil;
  if fmUSB.Memo1 <> nil then
    fmUSB.Memo1.Lines.Add('Thread terminated and nil');
end;

//========================= USB Thread Execute =================================
// Loop on USB Read; Write characters to USBCommBuffer
//==============================================================================

procedure TUSBthread.Execute;
var       i: Integer;
          Data: Byte;
begin
  fmDebug.AddDebugItem( 'Thread Running ' );
  ThreadStr := 'Thread Running';
  fmUSB.USBThreadRunning := True;
  Synchronize(UpdateMemo);
    while fmUSB.Tmo > 0 do begin  //1
      If Terminated then
        exit;
      Inc(LoopCount);
      if(_MPUSBRead(myInPipe,receive_buf,64,RecvLength,100)<>0) then begin //2
        // USB Read Successful
        for i := 0 to RecvLength-3 do begin //3
          Inc(TotalData);
          Data := receive_buf[i+2];
          fmUSB.WriteBuffer(Data,fmUSB.USBCommBuffer);
          Inc(DataCount); end;//-3
        fmUSB.Tmo := TMO_VAL;
        end //-2
      else  // USB Read timed out
        Dec(fmUSB.Tmo);
      end; //-1
    ThreadStr := 'Thread Timeout';
    Synchronize(UpdateMemo);
    fmUSB.USBThreadRunning := False;
  fmDebug.AddDebugItem( 'Thread Timeout ' );
end;

//==============================================================================
// ReceiveCal tries up to 10 100mSec timer ticks to receive entire string of 128 bytes
// If all bytes are received in CommBuffer the string is checked for error
// If array element 126 is a CR the string is OK and is written to the CalFile
// Otherwise CalError is set and Default Cal is used
//==============================================================================

function TfmUSB.USBReceiveCal( Var sData: Array of char ): Boolean;
var       ch1: BYTE;
          i: Integer;
          s: Array[0..127] of char;
          CalTimerCount: Integer;
begin
  fmDebug.AddDebugItem( 'USBReceiveCal ' );
  Result := False;
  If not USBconnected(ORTM4_VID_PID) then
    exit;
  USBStopSampling;
  DelayMsec(300);
  InitBuffer(USBCommBuffer);
  USBSendChar( char(CTRL_C) );
  DelayMsec(100);                   // Seems to be needed 3/14/09
  CalTimerCount := 10;
  While CalTimerCount > 0 do begin //1
    if USBCommBuffer.count >= 128 then begin //2 All data has been received
      CalTimerCount := 0; // Done
      ch1 := USBCommBuffer.buff[8];
      If ch1 = CR then begin //3
        Result := True;
        // Write PIC data to array
        for i := 0 to 127 do
          s[i] := char(USBCommBuffer.buff[i]);
        StrLCopy(sData, s, 128);
        end; //3
      end //2
    else begin //2 Waiting for data
      fmDebug.AddDebugItem( 'Waiting for Cal Data ' );
      dec( CalTimerCount );
      if CalTimerCount > 0 then
        DelayMsec(100);
      end; //2
    end; //1
  if not Result then
    USBCommOK := False;
  fmDebug.AddDebugItem( 'End USBReceiveCal ' );
end;

//==============================================================================
// Send calibration data to PIC memory in text file format with CRLFs
// Data is stored in string S and sent as a single string
// CTRL_D makes sure data transmission has stopped
// CTRL_E signals PIC to receive 128 bytes of data into PIC Program Flash Memory
// CTRL_D at end signals end of transmission
//==============================================================================

procedure TfmUSB.USBSendCal( Data: Array of Char );
var       S: String;
begin
  fmDebug.AddDebugItem( 'USBSendCal ' );
  If USBconnected(ORTM4_VID_PID) then begin //1
    S := char(CTRL_D) + char(CTRL_E);       // Start with two control characters
    S := S + Data;
    S := S + char(CTRL_D);
    // Total string length should be 2+128+1 = 131
    if( MessageDlg('Writing to PIC Memory',mtWarning,[mbOK,mbCancel], 0)) = mrOK
      then begin  //2
      USBSendStr( S );
      DelayMsec(300);                // Wait for transmission to complete
      end; //-2
    end; //-1
end;

//==============================================================================

procedure TfmUSB.InitBuffer(var buffer: tCommBuff);
var       i: word;
begin
  fmDebug.AddDebugItem( 'USB InitBuffer ' );
  for i := 0 to MAXBUFF-1 do
    buffer.buff[i] := 0;
  buffer.inptr := 0;
  buffer.outptr := 0;
  buffer.count := 0;
end;

function TfmUSB.WriteBuffer(uchar: BYTE; var buffer: tCommBuff): Boolean;
begin
if (buffer.count < MAXBUFF) then begin //1
  buffer.buff[buffer.inptr] := uchar;
  inc(buffer.inptr);
  if (buffer.inptr >= MAXBUFF) then
    buffer.inptr := 0;
  inc(buffer.count);
  result := True; end //-1
else
  result := False;
end;

function TfmUSB.ReadBuffer(var buffer: tCommBuff): BYTE;
var      uchar: Integer;
begin
if (buffer.count > 0) then begin //1
  uchar := buffer.buff[buffer.outptr];
  dec(buffer.count);
  inc(buffer.outptr);
  if (buffer.outptr >= MAXBUFF) then
    buffer.outptr := 0;
  result := uchar; end //-1
else
  result := 0;
end;

//============================== USB Timer Tick ================================
// Display errors while sampling
// was Loop through USBCommBuffer; write characters to Memo
//==============================================================================
procedure TfmUSB.USBTimerTimer(Sender: TObject);
var       i: word;
          ElapsedTime: real;
begin
  ebUSBError.Text := IntToStr(USBRxError);
  ebUSBRxData.Text := IntToStr(USBRxdCount);
  if USBRxDcount > USBMaxRxDcount then
    USBMaxRxDcount := USBRxDcount;
  ebUSBMaxRxData.Text := IntToStr(USBMaxRxdCount);
  if USBCommError > 0 then
    Memo1.Lines.Add( Format('Error at %f seconds', [TotalData/4800]));
    if (USBCommError AND E_FIRSTCHAR)>0 then
      Memo1.Lines.Add('Error Type = ' + 'FirstChar');
    if (USBCommError AND E_SECONDCHAR)>0 then
      Memo1.Lines.Add('Error Type = ' + 'SecondChar');
    if (USBCommError AND E_CHARMATCH)>0 then
      Memo1.Lines.Add('Error Type = ' + 'CharMatch');
    USBCommError := 0;
//  exit; // Bypass writing to Memo
  ElapsedTime := TotalData/4800;
  Edit2.Text := Format('%8.4f sec', [ElapsedTime]);

end;

function  TfmUSB.USBConnected(pid_vid: PCHAR8): Boolean;
begin
  if (_MPUSBGetDeviceCount(pid_vid)=0) then
    result := False
  else
    result := True;
end;

//============================ OpenUSB =========================================
// Set up and connect to the USB device. Return True if OK, False if not
//==============================================================================
function TfmUSB.OpenUSB: Boolean;
var      USBinfo: TUSBPortinfo;
begin
  fmDebug.AddDebugItem( 'OpenUSB ');
  USBTimer.Enabled := False; // Only while sampling
  if not USBConnected(ORTM4_VID_PID) then begin
    result := False;
    exit;
    end;

  if (myOutPipe <> INVALID_HANDLE_VALUE) AND (myInPipe <> INVALID_HANDLE_VALUE) then begin
    fmDebug.AddDebugItem( 'USB Handles valid' );
    Result := True; end
  else begin
    if myOutPipe <> INVALID_HANDLE_VALUE then
      _MPUSBClose(myOutPipe);
    if myInPipe <> INVALID_HANDLE_VALUE then
      _MPUSBClose(myInPipe);
    myInPipe:= INVALID_HANDLE_VALUE;
    myOutPipe:=INVALID_HANDLE_VALUE;
    myOutPipe:= _MPUSBOpen(selection, ORTM4_VID_PID, out_pipe, MP_WRITE, 0);
    myInPipe:= _MPUSBOpen(selection, ORTM4_VID_PID, in_pipe, MP_READ, 0);
    end;

  InitBuffer(USBCommBuffer);
  if ((myOutPipe = INVALID_HANDLE_VALUE) or (myInPipe = INVALID_HANDLE_VALUE)) then begin
    fmDebug.AddDebugItem( 'USB Error Invalid Handle' );
    Memo1.lines.add('USB Error Invalid Handle');
    result := False;
    exit; end;

  USBinfo := GetUSBPortInfo(ORTM4_VID_PID,DEVSN3);
  If USBinfo.ClassName <> '' then
    ComboBox1.Text := ORTM4_VID_PID + '\' + DEVSN3 + ' ' + USBinfo.ClassName;

  result := True;
end;

//============================ CloseUSB ========================================
// Disconnect from the USB device.
//==============================================================================
procedure TfmUSB.CloseUSB;
begin
  fmDebug.AddDebugItem( 'CloseUSB ');
  if myOutPipe <> INVALID_HANDLE_VALUE then
    _MPUSBClose(myOutPipe);
  if myInPipe <> INVALID_HANDLE_VALUE then
    _MPUSBClose(myInPipe);
  myInPipe:= INVALID_HANDLE_VALUE;
  myOutPipe:=INVALID_HANDLE_VALUE;
  USBCommOK := False;
  if thUSBread <> nil then
    thUSBread.Terminate;
end;

//======================== ReadBuffer ==========================================
// ReadBuffer converts two bytes to an integer representing current data
// The CommRead pointer and RxDataCount increments and the CommCharCount decrements
// Actual data 0-4095 is offset by 2048 for bipolar current
// Returns signed integer data
//==============================================================================

function TfmUSB.USBReadBuffer : Integer;
var
  ch1, ch2: BYTE;
  RecdData: Word;
  DataChk: SmallInt;

begin
     RecdData := 2048;  //Default for zero
     if USBCommBuffer.count >= 2 then begin //1
       USBCommCriticalSection.Enter;
       ch1 := ReadBuffer(USBCommBuffer);  // Read first character
       if ( (ch1 and $01) <> 0) then begin //2
         // Wrong sequence, exit and read next byte
         Inc(USBRxError);                    // Bit 0 of LSB must be 0
         USBCommError := USBCommError or E_FIRSTCHAR; //eFirstChar;
         result := 0;
         USBCommCriticalSection.Leave;
         exit;
         end; //-2

       ch2 := ReadBuffer(USBCommBuffer);  // Read second character
       if ( (ch2 and $01) <> 1) then begin //2
         // Wrong sequence, exit and read next byte
         Inc(USBRxError);                    // Bit 0 of LSB must be 0
         USBCommError := USBCommError or E_SECONDCHAR;  //eSecondChar;
         result := 0;
         USBCommCriticalSection.Leave;
         exit;
         end; //-2

       USBCommCriticalSection.Leave;
       DataChk := BYTE(ch1) AND $02;
       If (ch2 AND $02) <> DataChk then begin //2
         Inc(USBRxError);                        // Two low bits must be equal
         USBCommError := USBCommError or E_CHARMATCH;  //eCharMatch;
         Result := 0;
         end //-2
       else begin //2
         RecdData := BYTE(ch2) AND $FC;      // MSB
         RecdData := RecdData shl 6;
         RecdData := RecdData OR BYTE(ch1);  // LSB
         RecdData := RecdData AND $0FFC;     // 10 bit precision
         Result := Integer(RecdData) - 2048;   // Actual signed data
         end; //-2
       end //-1
     else // If CommCharCount < 2 (ERROR)
       Result := 0;                          // Default if error
end;

//==============================================================================
//************************ USBSendChar *******************************
//==============================================================================
function TfmUSB.USBSendChar( ch: char ): Boolean;
var  SentDataLength:DWORD ;
     SendLength: Integer;
begin
  Tmo := TMO_VAL;
  if thUSBread = nil then begin
    thUSBread := TUSBThread.Create(True);
    fThreadTerminate := thUSBread.SetNil;
    thUSBread.OnTerminate := fThreadTerminate;
    thUSBread.Priority := MyPriority; end;
  send_buf[0]:= $80;
  send_buf[1]:= 3;
  send_buf[2]:=BYTE(ch);
  SendLength := 3;
  thUSBread.Resume;
  if(_MPUSBWrite(myOutPipe,send_buf,SendLength,SentDataLength,100)<>0) then begin
    Tmo := TMO_VAL;
    thUSBread.Resume;
    result := True; end
  else
    result := False;
end;

//==============================================================================
//************************ USBSendRange *******************************
//==============================================================================
procedure TfmUSB.USBSendRange( RangeNum: Integer );
var RngChar: Char;
begin
  RngChar := Char( (8-RangeNum) + 48);  //Used to send range number to PIC
  fmDebug.AddDebugItem( 'USB Send Range '+ RngChar );
  if USBconnected(ORTM4_VID_PID) then begin
    if USBSampling then
      USBStopSampling;
    DelayMsec(100);
    if not USBSendChar(RngChar) then
      fmDebug.AddDebugItem( 'USB Send Range Error' );
    DelayMsec(100);
    USBStartSampling;
  end;
end;

//======================== USBSendStr ==========================================
// Sends a packet byte[0]=0, byte[1]=len, byte[2..63]=characters
//==============================================================================
function TfmUSB.USBSendStr( S: String ): Boolean;
var  SentDataLength:DWORD ;
     SendLength: Integer;
     SendStr: String;
     index: Integer;
begin
  fmDebug.AddDebugItem( 'USB Send String '+ S );
  SendLength := Length(S);
  While SendLength > 0 do begin //1
    if Length(S) > MAXSIZE-2 then
      SendStr := Copy(S, 1, MAXSIZE-2)
    else
      SendStr := S;
    SendLength := Length(SendStr);  //SendLength = actual chars to send
    if SendLength = 0 then
      break;
    send_buf[0]:= $F0;
    send_buf[1]:= SendLength+2; //includes CMD and len
    for index := 0 to SendLength-1 do
      send_buf[index+2] := BYTE(SendStr[index+1]);
    Memo1.Lines.Add(SendStr);
    if thUSBread = nil then begin //1
      thUSBread := TUSBThread.Create(True);
      fThreadTerminate := thUSBread.SetNil;
      thUSBread.OnTerminate := fThreadTerminate;
      thUSBread.Priority := MyPriority; end; //-1
    Tmo := TMO_VAL;
    thUSBread.Resume;
    DelayMsec(100);
    if(_MPUSBWrite(myOutPipe,send_buf,SendLength+2,SentDataLength,100)<>0) then begin
      Tmo := TMO_VAL;
      Result := True; end
    else begin
      fmDebug.AddDebugItem( 'USB SendStr Error ' );
      Result := False; end;
    S := Copy( S, SendLength+1, Length(S)-SendLength ); //SendLength is actual chars sent
    SendLength := Length(S); // SendLength = new length of S
    if SendLength > 0 then
      DelayMsec(300);
  end; //-1
end;

//======================== USBStartSampling =======================================
// Send character to start data transmission
//==============================================================================
procedure TfmUSB.USBStartSampling;
begin
  fmDebug.AddDebugItem( 'USB StartSampling ' );
  if USBConnected(ORTM4_VID_PID) then begin
    InitBuffer(USBCommBuffer);
    Tmo := TMO_VAL;
    if thUSBread = nil then begin //1
      thUSBread := TUSBThread.Create(True);
      fThreadTerminate := thUSBread.SetNil;
      thUSBread.OnTerminate := fThreadTerminate;
      thUSBread.Priority := MyPriority; end; //-1
    thUSBread.Resume;
    USBSendChar(Chr(Ctrl_A));        // Start transmission
    USBSampling := True;
    USBRxDerrorCount := 0;
    USBTimer.Enabled := True;
  end;
end;

//========================= StopSampling =======================================
// Send character to stop data transmission
//==============================================================================
procedure TfmUSB.USBStopSampling;
begin
  fmDebug.AddDebugItem( 'USB StopSampling ' );
  if USBConnected(ORTM4_VID_PID) then
    USBSendChar(Chr(Ctrl_D));        // Stop transmission
  USBSampling := False;
  if thUSBread <> nil then
    thUSBread.Terminate;
//  Delay(200);
  USBTimer.Enabled := False;
end;

//======================== TestUSBComm =========================================
// TUSBStatus = (uNone, uNotConnected, uOpenError, uTimeout, uCharError, uOK );
//==============================================================================

function    TfmUSB.USBTestComm: TUSBStatus;
var         S: String;
            TestByte: BYTE;
            Timeout, Tmo1: Integer;
begin
  fmDebug.AddDebugItem( 'USB TestComm ' );
  Result := uNone;
  if USBsampling then
     USBStopSampling;
  DelayMsec(100);
  InitBuffer(USBCommBuffer);
  if USBconnected(ORTM4_VID_PID) then begin
    if not OpenUSB then
      Result := uOpenError;
    end
  else
    Result := uNotConnected;

  if Result > uNone then
    exit;

  USBSendChar(char(CR));
  DelayMsec(100);
  fmDebug.AddDebugItem( 'USBCommBuffer.count='+IntToStr(USBCommBuffer.count) );
  Timeout := 10;
  fmDebug.AddDebugItem( 'Reading excess characters ' );
  while (Timeout > 0) and (USBCommBuffer.count > 0) do begin
    TestByte := ReadBuffer( USBCommBuffer );
    fmDebug.AddDebugItem( 'USBCommBuffer.count='+IntToStr(USBCommBuffer.count) );
    Dec(Timeout);
    end;
  Timeout := 10;
  S := 'A' + char(CR);
  USBSendStr(S);
  fmDebug.AddDebugItem( 'USB Sent test string ' );
  While Timeout > 0 do begin //1
    If USBCommBuffer.count > 1 then
      break
    else
      dec(Timeout);
      DelayMsec(100);
   end; //-1
  // USBCommBuffer contains only characters sent from PIC and not CMD or len
  fmDebug.AddDebugItem( 'USBCommBuffer.count='+IntToStr(USBCommBuffer.count) );
  if Timeout > 0 then begin //1
    TestByte := USBCommBuffer.buff[USBCommBuffer.outptr];
    fmDebug.AddDebugItem('USB Received test character ['+IntToStr(TestByte)+']' );
    If char(TestByte) = 'A' then begin //2
      Result := uOK;
      USBCommOK := True;
      end //-2
    else begin //2
      fmDebug.AddDebugItem( 'USB Wrong test char ['+IntToStr(TestByte)+']' );
      Result := uCharError;
      USBCommOK := False;
      end; //-2
    end //-1
  else begin //1
    fmDebug.AddDebugItem( 'USB Comm Test Timeout' );
    Result := uTimeout;
    USBCommOK := False;
    end; //-1
  InitBuffer(USBCommBuffer);
  fmDebug.AddDebugItem( 'End USBTestComm ' );
end;

procedure TfmUSB.FormCreate(Sender: TObject);
begin
  USBRxError := 0;
  USBCommCriticalSection := TCriticalSection.Create;
  MyPriority := tpTimeCritical; // tpHighest
end;

procedure TfmUSB.FormDestroy(Sender: TObject);
begin
  if USBCommCriticalSection <> nil then begin
    USBCommCriticalSection.Free;
    USBCommCriticalSection := nil;
    end;
end;

function   TfmUSB.GetUSBPortInfo( id: String; sn: String ): TUSBPortInfo;
var        S: String;
           Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  if Reg = nil then
    fmDebug.AddDebugItem( 'Error creating Reg ' );
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    S := 'SYSTEM\CurrentControlSet\Enum\USB';
    S := S + '\' + id + '\' + sn;           // Vid_04d8&Pid_fdcf\0123456789';
    fmDebug.AddDebugItem( 'Key: ' + S );
    try
      Reg.OpenKeyReadOnly(S);
      result.ClassName := Reg.ReadString('Class');
//      S := 'Device Parameters';
//      Reg.OpenKeyReadOnly(S);
      result.Service := Reg.ReadString('Service');
    except
      result.ClassName := '';
      result.Service := '';
    end;
  finally
    Reg.Free;
  end;
end;

procedure TfmUSB.FormShow(Sender: TObject);
begin
  fmUSB.Left := Screen.Width - fmUSB.Width;
end;

end.
